module SVG
  module Graph
    VERSION = '2.2.1'

  end
end
require_relative 'SVG/Graph/C3js'
require_relative 'SVG/Graph/Graph'
require_relative 'SVG/Graph/DataPoint'
require_relative 'SVG/Graph/BarBase'
require_relative 'SVG/Graph/Bar'
require_relative 'SVG/Graph/BarHorizontal'
require_relative 'SVG/Graph/ErrBar'
require_relative 'SVG/Graph/Line'
require_relative 'SVG/Graph/Pie'
require_relative 'SVG/Graph/Plot'
require_relative 'SVG/Graph/Schedule'
require_relative 'SVG/Graph/TimeSeries'
